@@ -14,7 +14,7 @@
 
 package Lemonldap::NG::Manager::Build::CTrees;
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.4';
 
 sub cTrees {
     return {
@@ -22,13 +22,15 @@ sub cTrees {
             'locationRules',
             'exportedHeaders',
             'post',
-            {   title => 'vhostOptions',
+            {
+                title => 'vhostOptions',
                 help  => 'configvhost.html#options',
                 form  => 'simpleInputContainer',
                 nodes => [
                     'vhostPort',        'vhostHttps',
                     'vhostMaintenance', 'vhostAliases',
                     'vhostType',        'vhostAuthnLevel',
+                    'vhostServiceTokenTTL'
                 ],
             },
         ],
@@ -36,7 +38,8 @@ sub cTrees {
             'samlIDPMetaDataXML',
             'samlIDPMetaDataExportedAttributes',
 
-            {   title => "samlIDPMetaDataOptionsSession",
+            {
+                title => "samlIDPMetaDataOptionsSession",
                 form  => 'simpleInputContainer',
                 nodes => [
                     "samlIDPMetaDataOptionsAdaptSessionUtime",
@@ -45,7 +48,8 @@ sub cTrees {
                     "samlIDPMetaDataOptionsUserAttribute"
                 ]
             },
-            {   title => "samlIDPMetaDataOptionsSignature",
+            {
+                title => "samlIDPMetaDataOptionsSignature",
                 form  => 'simpleInputContainer',
                 nodes => [
                     "samlIDPMetaDataOptionsSignSSOMessage",
@@ -54,14 +58,16 @@ sub cTrees {
                     "samlIDPMetaDataOptionsCheckSLOMessageSignature"
                 ]
             },
-            {   title => "samlIDPMetaDataOptionsBinding",
+            {
+                title => "samlIDPMetaDataOptionsBinding",
                 form  => 'simpleInputContainer',
                 nodes => [
                     "samlIDPMetaDataOptionsSSOBinding",
                     "samlIDPMetaDataOptionsSLOBinding"
                 ]
             },
-            {   title => "samlIDPMetaDataOptionsSecurity",
+            {
+                title => "samlIDPMetaDataOptionsSecurity",
                 form  => 'simpleInputContainer',
                 nodes => [
                     "samlIDPMetaDataOptionsEncryptionMode",
@@ -69,7 +75,8 @@ sub cTrees {
                     "samlIDPMetaDataOptionsCheckAudience"
                 ]
             },
-            {   title => 'samlIDPMetaDataOptions',
+            {
+                title => 'samlIDPMetaDataOptions',
                 help  => 'authsaml.html#options',
                 form  => 'simpleInputContainer',
                 nodes => [
@@ -83,21 +90,24 @@ sub cTrees {
                     'samlIDPMetaDataOptionsRelayStateURL',
                 ],
             },
-            {   title => "samlIDPMetaDataOptionsDisplay",
+            {
+                title => "samlIDPMetaDataOptionsDisplay",
                 form  => 'simpleInputContainer',
                 nodes => [
                     "samlIDPMetaDataOptionsDisplayName",
-                    "samlIDPMetaDataOptionsIcon"
+                    "samlIDPMetaDataOptionsIcon",
+                    "samlIDPMetaDataOptionsSortNumber"
                 ]
             }
         ],
         samlSPMetaDataNode => [
             "samlSPMetaDataXML",
             "samlSPMetaDataExportedAttributes",
-            {   title => "samlSPMetaDataOptions",
+            {
+                title => "samlSPMetaDataOptions",
                 help  => 'idpsaml.html#options',
-                nodes => [
-                    {   title => "samlSPMetaDataOptionsAuthnResponse",
+                nodes => [ {
+                        title => "samlSPMetaDataOptionsAuthnResponse",
                         form  => 'simpleInputContainer',
                         nodes => [
                             "samlSPMetaDataOptionsNameIDFormat",
@@ -108,7 +118,8 @@ sub cTrees {
                             "samlSPMetaDataOptionsForceUTF8"
                         ]
                     },
-                    {   title => "samlSPMetaDataOptionsSignature",
+                    {
+                        title => "samlSPMetaDataOptionsSignature",
                         form  => 'simpleInputContainer',
                         nodes => [
                             "samlSPMetaDataOptionsSignSSOMessage",
@@ -117,7 +128,8 @@ sub cTrees {
                             "samlSPMetaDataOptionsCheckSLOMessageSignature"
                         ]
                     },
-                    {   title => "samlSPMetaDataOptionsSecurity",
+                    {
+                        title => "samlSPMetaDataOptionsSecurity",
                         form  => 'simpleInputContainer',
                         nodes => [
                             "samlSPMetaDataOptionsEncryptionMode",
@@ -132,9 +144,10 @@ sub cTrees {
             'oidcOPMetaDataJSON',
             'oidcOPMetaDataJWKS',
             'oidcOPMetaDataExportedVars',
-            {   title => 'oidcOPMetaDataOptions',
-                nodes => [
-                    {   title => 'oidcOPMetaDataOptionsConfiguration',
+            {
+                title => 'oidcOPMetaDataOptions',
+                nodes => [ {
+                        title => 'oidcOPMetaDataOptionsConfiguration',
                         form  => 'simpleInputContainer',
                         nodes => [
                             'oidcOPMetaDataOptionsConfigurationURI',
@@ -144,7 +157,8 @@ sub cTrees {
                             'oidcOPMetaDataOptionsStoreIDToken'
                         ]
                     },
-                    {   title => 'oidcOPMetaDataOptionsProtocol',
+                    {
+                        title => 'oidcOPMetaDataOptionsProtocol',
                         form  => 'simpleInputContainer',
                         nodes => [
                             'oidcOPMetaDataOptionsScope',
@@ -161,24 +175,29 @@ sub cTrees {
                     },
                 ]
             },
-            {   title => 'oidcOPMetaDataOptionsDisplayParams',
+            {
+                title => 'oidcOPMetaDataOptionsDisplayParams',
                 form  => 'simpleInputContainer',
                 nodes => [
                     'oidcOPMetaDataOptionsDisplayName',
-                    'oidcOPMetaDataOptionsIcon'
+                    'oidcOPMetaDataOptionsIcon',
+                    'oidcOPMetaDataOptionsSortNumber'
                 ]
             },
         ],
         oidcRPMetaDataNode => [
             'oidcRPMetaDataExportedVars',
             'oidcRPMetaDataOptionsExtraClaims',
-            {   title => 'oidcRPMetaDataOptions',
-                nodes => [
-                    {   title => 'oidcRPMetaDataOptionsAuthentication',
+            {
+                title => 'oidcRPMetaDataOptions',
+                nodes => [ {
+                        title => 'oidcRPMetaDataOptionsAuthentication',
                         form  => 'simpleInputContainer',
                         nodes => [
                             'oidcRPMetaDataOptionsClientID',
-                            'oidcRPMetaDataOptionsClientSecret'
+                            'oidcRPMetaDataOptionsClientSecret',
+                            'oidcRPMetaDataOptionsPublic',
+                            'oidcRPMetaDataOptionsRequirePKCE',
                         ]
                     },
                     'oidcRPMetaDataOptionsUserIDAttr',
@@ -187,7 +206,8 @@ sub cTrees {
                     'oidcRPMetaDataOptionsAccessTokenExpiration',
                     'oidcRPMetaDataOptionsRedirectUris',
                     'oidcRPMetaDataOptionsBypassConsent',
-                    {   title => 'logout',
+                    {
+                        title => 'logout',
                         form  => 'simpleInputContainer',
                         nodes => [
                             'oidcRPMetaDataOptionsPostLogoutRedirectUris',
@@ -199,7 +219,8 @@ sub cTrees {
                     'oidcRPMetaDataOptionsRule',
                 ]
             },
-            {   title => 'oidcRPMetaDataOptionsDisplay',
+            {
+                title => 'oidcRPMetaDataOptionsDisplay',
                 form  => 'simpleInputContainer',
                 nodes => [
                     'oidcRPMetaDataOptionsDisplayName',
@@ -210,7 +231,8 @@ sub cTrees {
         casSrvMetaDataNode => [
             'casSrvMetaDataExportedVars',
             'casSrvMetaDataOptionsProxiedServices',
-            {   title => 'casSrvMetaDataOptions',
+            {
+                title => 'casSrvMetaDataOptions',
                 form  => 'simpleInputContainer',
                 nodes => [
                     'casSrvMetaDataOptionsUrl',
@@ -218,19 +240,22 @@ sub cTrees {
                     'casSrvMetaDataOptionsGateway',
                 ]
             },
-            {   title => 'casSrvMetaDataOptionsDisplay',
+            {
+                title => 'casSrvMetaDataOptionsDisplay',
                 form  => 'simpleInputContainer',
                 nodes => [
                     'casSrvMetaDataOptionsDisplayName',
                     'casSrvMetaDataOptionsIcon',
+                    'casSrvMetaDataOptionsSortNumber',
                 ]
             },
         ],
-        casAppMetaDataNode => [
-            {   title => 'casAppMetaDataOptions',
+        casAppMetaDataNode => [ {
+                title => 'casAppMetaDataOptions',
                 form  => 'simpleInputContainer',
                 nodes => [
                     'casAppMetaDataOptionsService',
+                    'casAppMetaDataOptionsUserAttribute',
                     'casAppMetaDataOptionsRule'
                 ]
             },
