@@ -20,12 +20,13 @@ package Lemonldap::NG::Manager::Conf::Parser;
 
 use strict;
 use utf8;
+use Crypt::URandom;
 use Mouse;
 use JSON 'to_json';
 use Lemonldap::NG::Common::Conf::ReConstants;
 use Lemonldap::NG::Manager::Attributes;
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.4';
 
 extends 'Lemonldap::NG::Common::Conf::Compact';
 
@@ -55,10 +56,10 @@ has message => (
         hdebug( "Message becomes " . $_[0]->{message} );
     }
 );
+has needConfirmation =>
+  ( is => 'rw', isa => 'ArrayRef', default => sub { return [] } );
 
 # Booleans
-has needConfirm =>
-  ( is => 'rw', isa => 'ArrayRef', default => sub { return [] } );
 has confChanged => (
     is      => 'rw',
     isa     => 'Bool',
@@ -125,13 +126,14 @@ sub scanTree {
     # metadata and set a value to the key if empty
     $self->newConf->{cfgNum} = $self->req->params('cfgNum');
     $self->newConf->{cfgAuthor} =
-      $self->req->userData->{ $Lemonldap::NG::Handler::Main::tsv->{whatToTrace}
+      $self->req->userData->{ Lemonldap::NG::Handler::Main->tsv->{whatToTrace}
           || '_whatToTrace' } // "anonymous";
     $self->newConf->{cfgAuthorIP} = $self->req->address;
     $self->newConf->{cfgDate}     = time;
-    $self->newConf->{cfgVersion}  = $VERSION;
-    $self->newConf->{key} ||=
-      join( '', map { chr( int( rand(94) ) + 33 ) } ( 1 .. 16 ) );
+    $self->newConf->{cfgVersion}  = $Lemonldap::NG::Manager::VERSION;
+    $self->newConf->{key} ||= join( '',
+        map { chr( int( ord( Crypt::URandom::urandom(1) ) * 94 / 256 ) + 33 ) }
+          ( 1 .. 16 ) );
 
     return 1;
 }
@@ -306,7 +308,8 @@ sub _scanNodes {
             # SAML
             elsif ( $base =~ /^saml(?:S|ID)PMetaDataNodes$/ ) {
                 hdebug('SAML');
-                if ( defined $leaf->{data} and ref( $leaf->{data} ) eq 'ARRAY' )
+                if ( defined $leaf->{data}
+                    and ref( $leaf->{data} ) eq 'ARRAY' )
                 {
                     hdebug("  SAML data is an array, serializing");
                     $leaf->{data} = join ';', @{ $leaf->{data} };
@@ -532,17 +535,25 @@ sub _scanNodes {
                         : {}
                     }
                   );
-                for ( my $i = 0 ; $i < @listCatNew ; $i++ ) {
-                    if ( not( defined $listCatRef[$i] )
-                        or $listCatRef[$i] ne $listCatNew[$i] )
-                    {
-                        push @{ $self->changes },
-                          {
-                            key => $leaf->{id},
-                            new => $listCatNew[$i],
-                            old => $listCatRef[$i]
-                          };
-                    }
+
+                @listCatRef = map { $_ ? $_ : () } @listCatRef;
+                @listCatNew = map { $_ ? $_ : () } @listCatNew;
+                @listCatRef = sort @listCatRef;
+                @listCatNew = sort @listCatNew;
+                hdebug( '# @listCatRef : ', \@listCatRef );
+                hdebug( '# @listCatNew : ', \@listCatNew );
+
+                # Check for deleted
+                my @diff =
+                  grep !${ { map { $_, 1 } @listCatNew } }{$_}, @listCatRef;
+                if ( scalar @diff ) {
+                    $self->confChanged(1);
+                    push @{ $self->changes },
+                      {
+                        new => join( ', ', 'categoryList',      @listCatNew ),
+                        key => join( ', ', 'Deletes in cat(s)', @diff ),
+                        old => join( ', ', 'categoryList',      @listCatRef ),
+                      };
                 }
             }
             next;
@@ -576,6 +587,22 @@ sub _scanNodes {
                 $cmp = $cmp->{$cat};
             }
 
+            my $newapp = $app;
+
+         # Compute a nice name for new nodes, taking care of potential conflicts
+         # For some reason, the manager sends /nNaN sometimes
+            if ( $newapp =~ /^n(\d+|NaN)$/ ) {
+
+                # Remove all special characters
+                my $baseName = $leaf->{title} =~ s/\W//gr;
+                $baseName = lc $baseName;
+                $newapp   = $baseName;
+                my $cnt = 1;
+                while ( exists $cn->{$newapp} ) {
+                    $newapp = "${baseName}_" . $cnt++;
+                }
+            }
+
             # Create new category
             #
             # Note that this works because nodes are ordered so "cat/cat2/app"
@@ -583,12 +610,14 @@ sub _scanNodes {
             if ( $leaf->{type} eq 'menuCat' ) {
                 hdebug('  menu cat');
                 $knownCat->{__id}++;
-                my $s = $knownCat->{$app} = sprintf '%04d-cat',
-                  $knownCat->{__id};
-                $cn->{$s} =
-                  { catname => $leaf->{title}, type => 'category' };
+                $knownCat->{$app} = $newapp;
+                $cn->{$newapp}    = {
+                    catname => $leaf->{title},
+                    type    => 'category',
+                    order   => $knownCat->{__id}
+                };
                 unless ($cmp->{$app}
-                    and $cmp->{$app}->{catname} eq $cn->{$s}->{catname} )
+                    and $cmp->{$app}->{catname} eq $cn->{$newapp}->{catname} )
                 {
                     $self->confChanged(1);
                     push @{ $self->changes },
@@ -596,15 +625,17 @@ sub _scanNodes {
                         key => join(
                             ', ', 'applicationList', @path, $leaf->{title}
                         ),
-                        new => $cn->{$s}->{catname},
-                        old => ( $cn->{$s} ? $cn->{$s}->{catname} : undef )
+                        new => $cn->{$newapp}->{catname},
+                        old => (
+                            $cn->{$newapp} ? $cn->{$newapp}->{catname} : undef
+                        )
                       };
                 }
                 if ( ref $subNodes ) {
                     $self->_scanNodes($subNodes) or return 0;
                 }
                 my @listCatRef = keys %{ $cmp->{$app} };
-                my @listCatNew = keys %{ $cn->{$s} };
+                my @listCatNew = keys %{ $cn->{$newapp} };
 
                 # Check for deleted
                 unless ( @listCatRef == @listCatNew ) {
@@ -621,10 +652,12 @@ sub _scanNodes {
             else {
                 hdebug('  new app');
                 $knownCat->{__id}++;
-                my $name = sprintf( '%04d-app', $knownCat->{__id} );
-                $cn->{$name} =
-                  { type => 'application', options => $leaf->{data} };
-                $cn->{$name}->{options}->{name} = $leaf->{title};
+                $cn->{$newapp} = {
+                    type    => 'application',
+                    options => $leaf->{data},
+                    order   => $knownCat->{__id}
+                };
+                $cn->{$newapp}->{options}->{name} = $leaf->{title};
                 unless ( $cmp->{$app} ) {
                     $self->confChanged(1);
                     push @{ $self->changes },
@@ -634,9 +667,17 @@ sub _scanNodes {
                       };
                 }
                 else {
-                    foreach my $k ( keys %{ $cn->{$name}->{options} } ) {
+                    # Check for change in ordering
+                    if ( ( $cn->{$newapp}->{order} || 0 ) !=
+                        ( $cmp->{$newapp}->{order} || 0 ) )
+                    {
+                        $self->confChanged(1);
+                    }
+
+                    # Check for change in options
+                    foreach my $k ( keys %{ $cn->{$newapp}->{options} } ) {
                         unless ( $cmp->{$app}->{options}->{$k} eq
-                            $cn->{$name}->{options}->{$k} )
+                            $cn->{$newapp}->{options}->{$k} )
                         {
                             $self->confChanged(1);
                             push @{ $self->changes },
@@ -644,7 +685,7 @@ sub _scanNodes {
                                 key => join( ', ',
                                     'applicationList', @path,
                                     $leaf->{title},    $k ),
-                                new => $cn->{$name}->{options}->{$k},
+                                new => $cn->{$newapp}->{options}->{$k},
                                 old => $cmp->{$app}->{options}->{$k}
                               };
                         }
@@ -1063,8 +1104,7 @@ sub _unitTest {
                 my $msg    = $attr->{msgFail} // $type->{msgFail};
                 $res = 0
                   unless (
-                    $self->_execTest(
-                        {
+                    $self->_execTest( {
                             keyTest    => $attr->{keyTest} // $type->{keyTest},
                             keyMsgFail => $attr->{keyMsgFail}
                               // $type->{keyMsgFail},
@@ -1165,7 +1205,7 @@ sub _globalTest {
         eval {
             ( $res, $msg ) = $sub->();
             if ( $res == -1 ) {
-                push @{ $self->needConfirm }, { message => $msg };
+                push @{ $self->needConfirmation }, { message => $msg };
             }
             elsif ($res) {
                 if ($msg) {
