@@ -4,7 +4,7 @@ use utf8;
 use Lemonldap::NG::Common::Regexp;
 use Lemonldap::NG::Handler::Main;
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.0.3';
 
 ## @method hashref tests(hashref conf)
 # Return a hash ref where keys are the names of the tests and values
@@ -288,7 +288,7 @@ sub tests {
             return 1;
         },
 
-        # Error if session Activity Timeout is equal or lower than one minute
+    # Error if activity timeout interval is higher than session activity timeout
         timeoutActivityInterval => sub {
             return 1 unless ( defined $conf->{timeoutActivityInterval} );
             return ( 0,
@@ -404,6 +404,16 @@ sub tests {
             return ( $res, join( ', ', @msg ) );
         },
 
+        # Test if SAML private and public keys signature keys are set
+        samlSecretKeys => sub {
+            return 1 unless ( $conf->{issuerDBSAMLActivation} );
+            return ( 0,
+                'SAML service private and public keys signature must be set' )
+              unless ( $conf->{samlServicePrivateKeySig}
+                && $conf->{samlServicePublicKeySig} );
+            return 1;
+        },
+
         # Try to parse combination with declared modules
         checkCombinations => sub {
             return 1 unless ( $conf->{authentication} eq 'Combination' );
@@ -573,9 +583,12 @@ sub tests {
         # Error if external 2F Send or Validate command is missing
         ext2fCommands => sub {
             return 1 unless ( $conf->{ext2fActivation} );
-            return ( 0, "External 2F Send or Validate command must be set" )
-              unless ( defined $conf->{ext2FSendCommand}
-                && defined $conf->{ext2FValidateCommand} );
+            return ( 0, "External 2F Send command must be set" )
+              unless ( defined $conf->{ext2FSendCommand} );
+            unless ( defined $conf->{ext2fCodeActivation} ) {
+                return ( 0, "External 2F Validate command must be set" )
+                  unless ( defined $conf->{ext2FValidateCommand} );
+            }
 
             # Return
             return 1;
@@ -630,6 +643,16 @@ sub tests {
             return 1;
         },
 
+        # Warn if Impersonation is enabled without prefix
+        impersonationPrefix => sub {
+            return 1 unless ( $conf->{impersonationRule} );
+            return ( 1,
+                "Impersonation is enabled without real attributes prefix" )
+              unless ( $conf->{impersonationPrefix} );
+
+            # Return
+            return 1;
+        },
     };
 }
 
