@@ -11,7 +11,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_PP_PASSWORD_EXPIRED
 );
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.0.5';
 
 # Inheritance: UserDB::LDAP provides all needed ldap functions
 extends
@@ -42,25 +42,36 @@ sub authenticate {
     # Set the dn unless done before
     unless ( $req->data->{dn} ) {
         if ( my $tmp = $self->getUser($req) ) {
-            $self->setSecurity($req);
+            eval { $self->setSecurity($req) };
+            $self->logger->warn($@) if ($@);
             return $tmp;
         }
     }
 
     unless ( $req->data->{password} ) {
         $self->p->{user} = $req->userData->{_dn} = $req->data->{dn};
-        unless($self->p->{_passwordDB}) {
+        unless ( $self->p->{_passwordDB} ) {
             $self->logger->error('No password database configured, aborting');
             return PE_ERROR;
         }
         my $res = $self->p->{_passwordDB}->_modifyPassword( $req, 1 );
 
+        # Refresh entry
+        if ( $self->p->{_userDB}->getUser($req) != PE_OK ) {
+            $self->logger->error(
+                "Unable to refresh entry for " . $self->p->{user} );
+        }
+
+        $req->data->{noerror} = 1;
+        $self->setSecurity($req);
+
         # Security: never create session here
         return $res || PE_DONE;
     }
     my $res =
       $self->userBind( $req, $req->data->{dn},
         password => $req->data->{password} );
+    $self->setSecurity($req) if ( $res > PE_OK );
 
     # Remember password if password reset needed
     if (
