@@ -15,7 +15,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OIDC_SERVICE_NOT_ALLOWED
 );
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.0.5';
 
 extends 'Lemonldap::NG::Portal::Main::Issuer',
   'Lemonldap::NG::Portal::Lib::OpenIDConnect',
@@ -29,7 +29,7 @@ sub beforeAuth { 'exportRequestParameters' }
 
 use constant sessionKind => 'OIDCI';
 
-has rule => ( is => 'rw', default => sub { {} } );
+has rule => ( is => 'rw' );
 has configStorage => (
     is      => 'ro',
     lazy    => 1,
@@ -150,7 +150,7 @@ sub run {
             foreach my $param (
                 qw/response_type scope client_id state redirect_uri nonce
                 response_mode display prompt max_age ui_locales id_token_hint
-                login_hint acr_values request request_uri/
+                login_hint acr_values request request_uri code_challenge code_challenge_method/
               )
             {
                 if ( $req->param($param) ) {
@@ -198,8 +198,8 @@ sub run {
 "Override $_ OIDC param by value present in request parameter"
                     );
                     $oidc_request->{$_} = $request->{$_};
-                    $self->p->setHiddenFormValue( $req, $_, $request->{$_},
-                        '' );
+                    $self->p->setHiddenFormValue( $req, $_, $request->{$_}, '',
+                        0 );
                 }
             }
 
@@ -556,6 +556,7 @@ sub run {
                         }
                         $req->info(
                             $self->loadTemplate(
+                                $req,
                                 'oidcGiveConsent',
                                 params => {
                                     displayName => $display_name,
@@ -574,6 +575,24 @@ sub run {
             my $session_state =
               $self->createSessionState( $req->id, $client_id );
 
+            # Check if PKCE is required
+            if ( $self->conf->{oidcRPMetaDataOptions}->{$rp}
+                ->{oidcRPMetaDataOptionsRequirePKCE}
+                and !$oidc_request->{'code_challenge'} )
+            {
+                $self->userLogger->error(
+                    "Relying Party must use PKCE protection");
+                return $self->returnRedirectError(
+                    $req,
+                    $oidc_request->{'redirect_uri'},
+                    'invalid_request',
+                    "Code challenge is required",
+                    undef,
+                    $oidc_request->{'state'},
+                    ( $flow ne "authorizationcode" )
+                );
+            }
+
             # Authorization Code Flow
             if ( $flow eq "authorizationcode" ) {
 
@@ -586,6 +605,9 @@ sub run {
                         user_session_id => $req->id,
                         _utime          => time,
                         nonce           => $oidc_request->{'nonce'},
+                        code_challenge  => $oidc_request->{'code_challenge'},
+                        code_challenge_method =>
+                          $oidc_request->{'code_challenge_method'},
                     }
                 );
 
@@ -855,7 +877,8 @@ sub run {
                     $self->logger->debug( "OIDC request parameter $param: "
                           . $oidc_request->{$param} );
                     $self->p->setHiddenFormValue( $req, $param,
-                        $oidc_request->{$param}, '' );
+                        $oidc_request->{$param},
+                        '', 0 );
                 }
             }
 
@@ -925,7 +948,7 @@ sub run {
                   : PE_OK;
             }
 
-            $req->info( $self->loadTemplate('oidcLogout') );
+            $req->info( $self->loadTemplate( $req, 'oidcLogout' ) );
             $req->data->{activeTimer} = 0;
             return PE_CONFIRM;
         }
@@ -943,7 +966,7 @@ sub token {
     my ( $client_id, $client_secret ) =
       $self->getEndPointAuthenticationCredentials($req);
 
-    unless ( $client_id && $client_secret ) {
+    unless ($client_id) {
         $self->logger->error(
 "No authentication provided to get token, or authentication type not supported"
         );
@@ -963,16 +986,35 @@ sub token {
     }
 
     # Check client_secret
-    unless ( $client_secret eq $self->conf->{oidcRPMetaDataOptions}->{$rp}
-        ->{oidcRPMetaDataOptionsClientSecret} )
+    if ( $self->conf->{oidcRPMetaDataOptions}->{$rp}
+        ->{oidcRPMetaDataOptionsPublic} )
     {
-        $self->logger->error("Wrong credentials for $rp");
-        return $self->p->sendError( 'invalid_request', 400 );
+        $self->logger->debug(
+            "Relying Party $rp is public, do not check client secret");
+    }
+    else {
+        unless ($client_secret) {
+            $self->logger->error(
+"Relying Party $rp is confidential but no client secret was provided to authenticate on token endpoint"
+            );
+            return $self->p->sendError( $req, 'invalid_request', 400 );
+        }
+        unless ( $client_secret eq $self->conf->{oidcRPMetaDataOptions}->{$rp}
+            ->{oidcRPMetaDataOptionsClientSecret} )
+        {
+            $self->logger->error("Wrong credentials for $rp");
+            return $self->p->sendError( $req, 'invalid_request', 400 );
+        }
     }
 
     # Get code session
     my $code = $req->param('code');
 
+    unless ($code) {
+        $self->logger->error("No code found on token endpoint");
+        return $self->p->sendError( $req, 'invalid_request', 400 );
+    }
+
     $self->logger->debug("OpenID Connect Code: $code");
 
     my $codeSession = $self->getOpenIDConnectSession($code);
@@ -982,11 +1024,27 @@ sub token {
         return $self->p->sendError( $req, 'invalid_request', 400 );
     }
 
+    # Check PKCE
+    if ( $self->conf->{oidcRPMetaDataOptions}->{$rp}
+        ->{oidcRPMetaDataOptionsRequirePKCE} )
+    {
+        unless (
+            $self->validatePKCEChallenge(
+                $req->param('code_verifier'),
+                $codeSession->data->{'code_challenge'},
+                $codeSession->data->{'code_challenge_method'}
+            )
+          )
+        {
+            return $self->p->sendError( $req, 'invalid_grant', 400 );
+        }
+    }
+
     # Check we have the same redirect_uri value
     unless ( $req->param("redirect_uri") eq $codeSession->data->{redirect_uri} )
     {
         $self->userLogger->error( "Provided redirect_uri does not match "
-              . $codeSession->{redirect_uri} );
+              . $codeSession->data->{redirect_uri} );
         return $self->p->sendError( $req, 'invalid_request', 400 );
     }
 
@@ -1453,10 +1511,13 @@ sub metadata {
               [qw/none HS256 HS384 HS512 RS256 RS384 RS512/],
             userinfo_signing_alg_values_supported =>
               [qw/none HS256 HS384 HS512 RS256 RS384 RS512/],
+
+            # PKCE
+            code_challenge_methods_supported => [qw/plain S256/],
         }
     );
 
-    # response_modes_supported}
+    # response_modes_supported
 
     # id_token_encryption_alg_values_supported
     # id_token_encryption_enc_values_supported
