@@ -6,7 +6,7 @@ use strict;
 use Mouse;
 use Clone 'clone';
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.0.3';
 
 extends 'Lemonldap::NG::Common::Module';
 
@@ -136,17 +136,17 @@ sub displayModules {
             }
             elsif ( $module->[0] eq 'LoginHistory' ) {
                 $moduleHash->{'SUCCESS_LOGIN'} =
-                  $self->p->mkSessionArray(
+                  $self->p->mkSessionArray( $req,
                     $req->{sessionInfo}->{_loginHistory}->{successLogin},
                     "", 0, 0 );
                 $moduleHash->{'FAILED_LOGIN'} =
-                  $self->p->mkSessionArray(
+                  $self->p->mkSessionArray( $req,
                     $req->{sessionInfo}->{_loginHistory}->{failedLogin},
                     "", 0, 1 );
             }
             elsif ( $module->[0] eq 'OidcConsents' ) {
                 $moduleHash->{'OIDC_CONSENTS'} =
-                  $self->p->mkOidcConsent( $req->sessionInfo );
+                  $self->p->mkOidcConsent( $req, $req->sessionInfo );
             }
             push @$displayModules, $moduleHash;
         }
@@ -193,7 +193,7 @@ sub _buildCategoryHash {
     # Extract applications from hash
     my $apphash;
     foreach my $catkey ( sort keys %$cathash ) {
-        next if $catkey =~ /(type|options|catname)/;
+        next if $catkey =~ /(type|options|catname|order)/;
         if ( $cathash->{$catkey}->{type} eq "application" ) {
             $apphash->{$catkey} = $cathash->{$catkey};
         }
@@ -201,15 +201,31 @@ sub _buildCategoryHash {
 
     # Display applications first
     if ( scalar keys %$apphash > 0 ) {
-        foreach my $appkey ( sort keys %$apphash ) {
+        foreach my $appkey (
+            sort {
+                ( $apphash->{$a}->{order} || 0 )
+                  <=> ( $apphash->{$b}->{order} || 0 )
+                  or $a cmp $b
+            }
+            keys %$apphash
+          )
+        {
             push @$applications,
               $self->_buildApplicationHash( $appkey, $apphash->{$appkey} );
         }
     }
 
     # Display subcategories
-    foreach my $catkey ( sort keys %$cathash ) {
-        next if $catkey =~ /(type|options|catname)/;
+    foreach my $catkey (
+        sort {
+            ( $cathash->{$a}->{order} || 0 )
+              <=> ( $cathash->{$b}->{order} || 0 )
+              or $a cmp $b
+        }
+        grep { not /^(?:catname|type|options|order)$/ } keys %$cathash
+      )
+    {
+
         if ( $cathash->{$catkey}->{type} eq "category" ) {
             push @$categories,
               $self->_buildCategoryHash( $req, $catkey, $cathash->{$catkey},
@@ -246,7 +262,7 @@ sub _buildApplicationHash {
     # Detect sub applications
     my $subapphash;
     foreach my $key ( sort keys %$apphash ) {
-        next if $key =~ /(type|options|catname)/;
+        next if $key =~ /(type|options|catname|order)/;
         if ( $apphash->{$key}->{type} eq "application" ) {
             $subapphash->{$key} = $apphash->{$key};
         }
@@ -254,7 +270,15 @@ sub _buildApplicationHash {
 
     # Display sub applications
     if ( scalar keys %$subapphash > 0 ) {
-        foreach my $appkey ( sort keys %$subapphash ) {
+        foreach my $appkey (
+            sort {
+                ( $subapphash->{$a}->{order} || 0 )
+                  <=> ( $subapphash->{$b}->{order} || 0 )
+                  or $a cmp $b
+            }
+            keys %$subapphash
+          )
+        {
             push @$applications,
               $self->_buildApplicationHash( $appkey, $subapphash->{$appkey} );
         }
@@ -305,7 +329,7 @@ sub _filterHash {
     my ( $self, $req, $apphash ) = @_;
 
     foreach my $key ( keys %$apphash ) {
-        next if $key =~ /(type|options|catname)/;
+        next if $key =~ /(type|options|catname|order)/;
         if (    $apphash->{$key}->{type}
             and $apphash->{$key}->{type} eq "category" )
         {
@@ -319,7 +343,7 @@ sub _filterHash {
 
             # Find sub applications and filter them
             foreach my $appkey ( keys %{ $apphash->{$key} } ) {
-                next if $appkey =~ /(type|options|catname)/;
+                next if $appkey =~ /(type|options|catname|order)/;
 
                 # We have sub elements, so we filter them
                 $self->_filterHash( $req, $apphash->{$key} );
@@ -359,11 +383,11 @@ sub _filterHash {
 
             # If a specific rule exists, get it from cache or compile it
             if ( $appdisplay !~ /^auto$/i ) {
-                if ( $self->specific->{$appuri} ) {
-                    $cond = $self->specific->{$appuri};
+                if ( $self->specific->{$key} ) {
+                    $cond = $self->specific->{$key};
                 }
                 else {
-                    $cond = $self->specific->{$appuri} =
+                    $cond = $self->specific->{$key} =
                       $self->p->HANDLER->buildSub(
                         $self->p->HANDLER->substitute($appdisplay) );
                 }
@@ -393,7 +417,7 @@ sub _isCategoryEmpty {
 
     # Test sub categories
     foreach $key ( keys %$apphash ) {
-        next if $key =~ /(type|options|catname)/;
+        next if $key =~ /(type|options|catname|order)/;
         if (    $apphash->{$key}->{type}
             and $apphash->{$key}->{type} eq "category" )
         {
@@ -408,10 +432,12 @@ sub _isCategoryEmpty {
         # Temporary store 'options'
         my $tmp_options = $apphash->{options};
         my $tmp_catname = $apphash->{catname};
+        my $tmp_order   = $apphash->{order};
 
         delete $apphash->{type};
         delete $apphash->{options};
         delete $apphash->{catname};
+        delete $apphash->{order};
 
         if ( scalar( keys %$apphash ) ) {
 
@@ -420,6 +446,7 @@ sub _isCategoryEmpty {
             $apphash->{type}    = "category";
             $apphash->{options} = $tmp_options;
             $apphash->{catname} = $tmp_catname;
+            $apphash->{order}   = $tmp_order;
 
             # Return false
             return 0;
