@@ -123,6 +123,18 @@ sub clean_sessions {
     $cache->clear;
 }
 
+sub count_sessions {
+    my $dir = shift;
+    $dir ||= 't/sessions';
+    my $nbr = 0;
+
+    opendir D, $dir or die $!;
+    foreach ( grep { /^\w{64}$/ } readdir(D) ) {
+        $nbr++;
+    }
+    $nbr;
+}
+
 sub getCache {
     require Cache::FileCache;
     return Cache::FileCache->new( {
@@ -535,6 +547,11 @@ has ini => (
             cn   => 'Frédéric Accents',
             mail => 'fa@badwolf.org',
         };
+        $Lemonldap::NG::Portal::UserDB::Demo::demoAccounts{davros} = {
+            uid  => 'davros',
+            cn   => 'Bad Guy',
+            mail => 'davros@badguy.org',
+        };
         $Lemonldap::NG::Portal::UserDB::Demo::demoAccounts{russian} = {
             uid  => 'russian',
             cn   => 'Русский',
@@ -660,7 +677,7 @@ sub _get {
 Same as C<_get> except that a body is required. $body must be a file handle.
 Example with IO::String:
 
-  ok( 
+  ok(
       $res = $client->_post(
           '/',
           IO::String->new('user=dwho&password=dwho'),
